@@ -1,4 +1,4 @@
-from .form.utils import docval, call_docval_func
+from hdmf.utils import docval, call_docval_func
 from . import register_class, CORE_NAMESPACE
 from .core import NWBContainer
 
@@ -11,7 +11,6 @@ class Device(NWBContainer):
     __nwbfields__ = ('name',)
 
     @docval({'name': 'name', 'type': str, 'doc': 'the name of this device'},
-            {'name': 'source', 'type': str, 'doc': 'the source of the data'},
             {'name': 'parent', 'type': 'NWBContainer',
              'doc': 'The parent NWBContainer for this NWBContainer', 'default': None})
     def __init__(self, **kwargs):
