@@ -1,13 +1,14 @@
-import numpy as np
-from h5py import RegionReference
-from collections import Iterable
+try:
+    from collections.abc import Iterable  # Python 3
+except ImportError:
+    from collections import Iterable  # Python 2.7
 
-from .form.utils import docval, getargs, popargs, call_docval_func
-from .form.data_utils import DataChunkIterator, ShapeValidator
+from hdmf.utils import docval, getargs, popargs, call_docval_func
+from hdmf.data_utils import DataChunkIterator, assertEqualShape
 
 from . import register_class, CORE_NAMESPACE
 from .base import TimeSeries, _default_resolution, _default_conversion
-from .core import NWBContainer, NWBTable, NWBTableRegion, NWBDataInterface, MultiContainerInterface
+from .core import NWBContainer, NWBDataInterface, MultiContainerInterface, DynamicTableRegion
 from .device import Device
 
 
@@ -22,7 +23,6 @@ class ElectrodeGroup(NWBContainer):
                      'device')
 
     @docval({'name': 'name', 'type': str, 'doc': 'the name of this electrode'},
-            {'name': 'source', 'type': str, 'doc': 'the source of the data'},
             {'name': 'description', 'type': str, 'doc': 'description of this electrode group'},
             {'name': 'location', 'type': str, 'doc': 'description of location of this electrode group'},
             {'name': 'device', 'type': Device, 'doc': 'the device that was used to record from this electrode group'},
@@ -50,40 +50,6 @@ _et_docval = [
 ]
 
 
-@register_class('ElectrodeTable', CORE_NAMESPACE)
-class ElectrodeTable(NWBTable):
-    '''A table of all electrodes'''
-
-    __columns__ = _et_docval
-
-    @docval({'name': 'name', 'type': str, 'doc': 'the name of this container'},
-            {'name': 'data', 'type': ('array_data', 'data'), 'doc': 'the source of the data', 'default': list()})
-    def __init__(self, **kwargs):
-        data, name = getargs('data', 'name', kwargs)
-        colnames = [i['name'] for i in _et_docval]
-        super(ElectrodeTable, self).__init__(colnames, name, data)
-
-    @docval(*_et_docval)
-    def add_row(self, **kwargs):
-        kwargs['group_name'] = kwargs['group'].name
-        super(ElectrodeTable, self).add_row(kwargs)
-
-
-@register_class('ElectrodeTableRegion', CORE_NAMESPACE)
-class ElectrodeTableRegion(NWBTableRegion):
-    '''A subsetting of an ElectrodeTable'''
-
-    __nwbfields__ = ('description',)
-
-    @docval({'name': 'table', 'type': ElectrodeTable, 'doc': 'the ElectrodeTable this region applies to'},
-            {'name': 'region', 'type': (slice, list, tuple, RegionReference), 'doc': 'the indices of the table'},
-            {'name': 'description', 'type': str, 'doc': 'a brief description of what this electrode is'},
-            {'name': 'name', 'type': str, 'doc': 'the name of this container', 'default': 'electrodes'})
-    def __init__(self, **kwargs):
-        call_docval_func(super(ElectrodeTableRegion, self).__init__, kwargs)
-        self.description = getargs('description', kwargs)
-
-
 @register_class('ElectricalSeries', CORE_NAMESPACE)
 class ElectricalSeries(TimeSeries):
     """
@@ -92,27 +58,23 @@ class ElectricalSeries(TimeSeries):
     channels] (or [num_times] for single electrode).
     """
 
-    __nwbfields__ = ({'name': 'electrodes',
+    __nwbfields__ = ({'name': 'electrodes', 'required_name': 'electrodes',
                       'doc': 'the electrodes that generated this electrical series', 'child': True},)
 
-    __ancestry = "TimeSeries,ElectricalSeries"
     __help = "Stores acquired voltage data from extracellular recordings."
 
     @docval({'name': 'name', 'type': str, 'doc': 'The name of this TimeSeries dataset'},
-            {'name': 'source', 'type': str,
-             'doc': ('Name of TimeSeries or Modules that serve as the source for the data '
-                     'contained here. It can also be the name of a device, for stimulus or '
-                     'acquisition data')},
             {'name': 'data', 'type': ('array_data', 'data', TimeSeries),
+             'shape': ((None, ), (None, None), (None, None, None)),
              'doc': 'The data this TimeSeries dataset stores. Can also store binary data e.g. image frames'},
 
-            {'name': 'electrodes', 'type': ElectrodeTableRegion,
+            {'name': 'electrodes', 'type': DynamicTableRegion,
              'doc': 'the table region corresponding to the electrodes from which this series was recorded'},
             {'name': 'resolution', 'type': float,
              'doc': 'The smallest meaningful difference (in specified unit) between values in data',
              'default': _default_resolution},
             {'name': 'conversion', 'type': float,
-             'doc': 'Scalar to multiply each element by to conver to volts', 'default': _default_conversion},
+             'doc': 'Scalar to multiply each element by to convert to volts', 'default': _default_conversion},
 
             {'name': 'timestamps', 'type': ('array_data', 'data', TimeSeries),
              'doc': 'Timestamps for samples stored in data', 'default': None},
@@ -130,8 +92,8 @@ class ElectricalSeries(TimeSeries):
             {'name': 'parent', 'type': 'NWBContainer',
              'doc': 'The parent NWBContainer for this NWBContainer', 'default': None})
     def __init__(self, **kwargs):
-        name, source, electrodes, data = popargs('name', 'source', 'electrodes', 'data', kwargs)
-        super(ElectricalSeries, self).__init__(name, source, data, 'volt', **kwargs)
+        name, electrodes, data = popargs('name', 'electrodes', 'data', kwargs)
+        super(ElectricalSeries, self).__init__(name, data, 'volt', **kwargs)
         self.electrodes = electrodes
 
 
@@ -149,25 +111,20 @@ class SpikeEventSeries(ElectricalSeries):
 
     __nwbfields__ = ()
 
-    __ancestry = "TimeSeries,ElectricalSeries,SpikeSeries"
     __help = "Snapshots of spike events from data."
 
     @docval({'name': 'name', 'type': str, 'doc': 'The name of this TimeSeries dataset'},
-            {'name': 'source', 'type': str,
-             'doc': ('Name of TimeSeries or Modules that serve as the source for the data '
-                     'contained here. It can also be the name of a device, for stimulus or '
-                     'acquisition data')},
             {'name': 'data', 'type': ('array_data', 'data', TimeSeries),
              'doc': 'The data this TimeSeries dataset stores. Can also store binary data e.g. image frames'},
             {'name': 'timestamps', 'type': ('array_data', 'data', TimeSeries),
              'doc': 'Timestamps for samples stored in data'},
-            {'name': 'electrodes', 'type': ElectrodeTableRegion,
+            {'name': 'electrodes', 'type': DynamicTableRegion,
              'doc': 'the table region corresponding to the electrodes from which this series was recorded'},
             {'name': 'resolution', 'type': float,
              'doc': 'The smallest meaningful difference (in specified unit) between values in data',
              'default': _default_resolution},
             {'name': 'conversion', 'type': float,
-             'doc': 'Scalar to multiply each element by to conver to volts', 'default': _default_conversion},
+             'doc': 'Scalar to multiply each element by to convert to volts', 'default': _default_conversion},
             {'name': 'comments', 'type': str,
              'doc': 'Human-readable comments about this TimeSeries dataset', 'default': 'no comments'},
             {'name': 'description', 'type': str,
@@ -179,7 +136,7 @@ class SpikeEventSeries(ElectricalSeries):
             {'name': 'parent', 'type': 'NWBContainer',
              'doc': 'The parent NWBContainer for this NWBContainer', 'default': None})
     def __init__(self, **kwargs):
-        name, source, data, electrodes = popargs('name', 'source', 'data', 'electrodes', kwargs)
+        name, data, electrodes = popargs('name', 'data', 'electrodes', kwargs)
         timestamps = getargs('timestamps', kwargs)
         if not (isinstance(data, TimeSeries) and isinstance(timestamps, TimeSeries)):
             if not (isinstance(data, DataChunkIterator) and isinstance(timestamps, DataChunkIterator)):
@@ -188,7 +145,7 @@ class SpikeEventSeries(ElectricalSeries):
             else:
                 # TODO: add check when we have DataChunkIterators
                 pass
-        super(SpikeEventSeries, self).__init__(name, source, data, electrodes, **kwargs)
+        super(SpikeEventSeries, self).__init__(name, data, electrodes, **kwargs)
 
 
 @register_class('EventDetection', CORE_NAMESPACE)
@@ -202,11 +159,10 @@ class EventDetection(NWBDataInterface):
                      'source_idx',
                      'times')
 
-    _help_statement = ("Description of how events were detected, such as voltage "
-                       "threshold, or dV/dT threshold, as well as relevant values.")
+    __help = ("Description of how events were detected, such as voltage "
+              "threshold, or dV/dT threshold, as well as relevant values.")
 
-    @docval({'name': 'source', 'type': str, 'doc': 'the source of the data'},
-            {'name': 'detection_method', 'type': str,
+    @docval({'name': 'detection_method', 'type': str,
              'doc': 'Description of how events were detected, such as voltage threshold, or dV/dT threshold, \
              as well as relevant values.'},
             {'name': 'source_electricalseries', 'type': ElectricalSeries, 'doc': 'The source electrophysiology data'},
@@ -218,9 +174,9 @@ class EventDetection(NWBDataInterface):
             {'name': 'times', 'type': ('array_data', 'data'), 'doc': 'Timestamps of events, in Seconds'},
             {'name': 'name', 'type': str, 'doc': 'the name of this container', 'default': 'EventDetection'})
     def __init__(self, **kwargs):
-        source, detection_method, source_electricalseries, source_idx, times = popargs(
-            'source', 'detection_method', 'source_electricalseries', 'source_idx', 'times', kwargs)
-        super(EventDetection, self).__init__(source, **kwargs)
+        detection_method, source_electricalseries, source_idx, times = popargs(
+            'detection_method', 'source_electricalseries', 'source_idx', 'times', kwargs)
+        super(EventDetection, self).__init__(**kwargs)
         self.detection_method = detection_method
         # do not set parent, since this is a link
         self.source_electricalseries = source_electricalseries
@@ -250,6 +206,7 @@ class EventWaveform(MultiContainerInterface):
 @register_class('Clustering', CORE_NAMESPACE)
 class Clustering(NWBDataInterface):
     """
+    DEPRECATED in favor of :py:meth:`~pynwb.misc.Units`.
     Specifies cluster event times and cluster metric for maximum ratio of
     waveform peak to RMS on any channel in cluster.
     """
@@ -261,23 +218,25 @@ class Clustering(NWBDataInterface):
         'times'
     )
 
-    __help = ("Clustered spike data, whether from automatic clustering "
+    __help = ("[DEPRECATED] Clustered spike data, whether from automatic clustering "
               "tools (eg, klustakwik) or as a result of manual sorting.")
 
-    @docval({'name': 'source', 'type': str, 'doc': 'The source of the data'},
-            {'name': 'description', 'type': str,
+    @docval({'name': 'description', 'type': str,
              'doc': 'Description of clusters or clustering, (e.g. cluster 0 is noise, \
              clusters curated using Klusters, etc).'},
-            {'name': 'num', 'type': ('array_data', 'data'), 'doc': 'Cluster number of each event.'},
-            {'name': 'peak_over_rms', 'type': Iterable,
+            {'name': 'num', 'type': ('array_data', 'data'), 'doc': 'Cluster number of each event.', 'shape': (None, )},
+            {'name': 'peak_over_rms', 'type': Iterable, 'shape': (None, ),
              'doc': 'Maximum ratio of waveform peak to RMS on any channel in the cluster\
              (provides a basic clustering metric).'},
-            {'name': 'times', 'type': ('array_data', 'data'), 'doc': 'Times of clustered events, in seconds.'},
+            {'name': 'times', 'type': ('array_data', 'data'), 'doc': 'Times of clustered events, in seconds.',
+             'shape': (None,)},
             {'name': 'name', 'type': str, 'doc': 'the name of this container', 'default': 'Clustering'})
     def __init__(self, **kwargs):
-        source, description, num, peak_over_rms, times = popargs(
-            'source', 'description', 'num', 'peak_over_rms', 'times', kwargs)
-        super(Clustering, self).__init__(source, **kwargs)
+        import warnings
+        warnings.warn("use pynwb.misc.Units or NWBFile.units instead", DeprecationWarning)
+        description, num, peak_over_rms, times = popargs(
+            'description', 'num', 'peak_over_rms', 'times', kwargs)
+        super(Clustering, self).__init__(**kwargs)
         self.description = description
         self.num = num
         self.peak_over_rms = list(peak_over_rms)
@@ -287,6 +246,11 @@ class Clustering(NWBDataInterface):
 @register_class('ClusterWaveforms', CORE_NAMESPACE)
 class ClusterWaveforms(NWBDataInterface):
     """
+    DEPRECATED. `ClusterWaveforms` was deprecated in Oct 27, 2018 and will be removed in a future release.
+    Please use the `Units` table to store waveform mean and standard deviation
+    e.g. `NWBFile.units.add_unit(..., waveform_mean=..., waveform_sd=...)`
+
+
     Describe cluster waveforms by mean and standard deviation for at each sample.
     """
 
@@ -295,22 +259,25 @@ class ClusterWaveforms(NWBDataInterface):
                      'waveform_mean',
                      'waveform_sd')
 
-    __help = ("Mean waveform shape of clusters. Waveforms should be "
+    __help = ("[DEPRECATED] Mean waveform shape of clusters. Waveforms should be "
               "high-pass filtered (ie, not the same bandpass filter "
               "used waveform analysis and clustering)")
 
-    @docval({'name': 'source', 'type': str, 'doc': 'the source of the data'},
-            {'name': 'clustering_interface', 'type': Clustering,
+    @docval({'name': 'clustering_interface', 'type': Clustering,
              'doc': 'the clustered spike data used as input for computing waveforms'},
             {'name': 'waveform_filtering', 'type': str,
              'doc': 'filter applied to data before calculating mean and standard deviation'},
-            {'name': 'waveform_mean', 'type': Iterable, 'doc': 'the mean waveform for each cluster'},
-            {'name': 'waveform_sd', 'type': Iterable, 'doc': 'the standard deviations of waveforms for each cluster'},
+            {'name': 'waveform_mean', 'type': Iterable, 'shape': (None, None),
+             'doc': 'the mean waveform for each cluster'},
+            {'name': 'waveform_sd', 'type': Iterable, 'shape': (None, None),
+            'doc': 'the standard deviations of waveforms for each cluster'},
             {'name': 'name', 'type': str, 'doc': 'the name of this container', 'default': 'ClusterWaveforms'})
     def __init__(self, **kwargs):
-        source, clustering_interface, waveform_filtering, waveform_mean, waveform_sd = popargs(
-            'source', 'clustering_interface', 'waveform_filtering', 'waveform_mean', 'waveform_sd', kwargs)
-        super(ClusterWaveforms, self).__init__(source, **kwargs)
+        import warnings
+        warnings.warn("use pynwb.misc.Units or NWBFile.units instead", DeprecationWarning)
+        clustering_interface, waveform_filtering, waveform_mean, waveform_sd = popargs(
+            'clustering_interface', 'waveform_filtering', 'waveform_mean', 'waveform_sd', kwargs)
+        super(ClusterWaveforms, self).__init__(**kwargs)
         self.clustering_interface = clustering_interface
         self.waveform_filtering = waveform_filtering
         self.waveform_mean = waveform_mean
@@ -325,13 +292,12 @@ class LFP(MultiContainerInterface):
     ElectricalSeries description or comments field.
     """
 
-    __clsconf__ = {
-        'attr': 'electrical_series',
-        'type': ElectricalSeries,
-        'add': 'add_electrical_series',
-        'get': 'get_electrical_series',
-        'create': 'create_electrical_series',
-    }
+    __clsconf__ = [
+        {'attr': 'electrical_series',
+         'type': ElectricalSeries,
+         'add': 'add_electrical_series',
+         'get': 'get_electrical_series',
+         'create': 'create_electrical_series'}]
 
     __help = ("LFP data from one or more channels. Filter properties "
               "should be noted in the ElectricalSeries")
@@ -377,47 +343,46 @@ class FeatureExtraction(NWBDataInterface):
 
     __help = "Container for salient features of detected events"
 
-    @docval({'name': 'source', 'type': str, 'doc': 'The source of the data'},
-            {'name': 'electrodes', 'type': ElectrodeTableRegion,
+    @docval({'name': 'electrodes', 'type': DynamicTableRegion,
              'doc': 'the table region corresponding to the electrodes from which this series was recorded'},
-            {'name': 'description', 'type': (list, tuple, np.ndarray, DataChunkIterator),
-             'doc': 'A description for each feature extracted', 'ndim': 1},
-            {'name': 'times', 'type': ('array_data', 'data'),
-             'doc': 'The times of events that features correspond to', 'ndim': 1},
-            {'name': 'features', 'type': ('array_data', 'data'),
-             'doc': 'Features for each channel', 'ndim': 3},
+            {'name': 'description', 'type': ('array_data', 'data'),
+             'doc': 'A description for each feature extracted', 'shape': (None, )},
+            {'name': 'times', 'type': ('array_data', 'data'), 'shape': (None, ),
+             'doc': 'The times of events that features correspond to'},
+            {'name': 'features', 'type': ('array_data', 'data'), 'shape': (None, None, None),
+             'doc': 'Features for each channel'},
             {'name': 'name', 'type': str, 'doc': 'the name of this container', 'default': 'FeatureExtraction'})
     def __init__(self, **kwargs):
         # get the inputs
-        source, electrodes, description, times, features = popargs(
-            'source', 'electrodes', 'description', 'times', 'features', kwargs)
+        electrodes, description, times, features = popargs(
+            'electrodes', 'description', 'times', 'features', kwargs)
 
         # Validate the shape of the inputs
         # Validate event times compared to features
         shape_validators = []
-        shape_validators.append(ShapeValidator.assertEqualShape(data1=features,
-                                                                data2=times,
-                                                                axes1=0,
-                                                                axes2=0,
-                                                                name1='feature_shape',
-                                                                name2='times',
-                                                                ignore_undetermined=True))
+        shape_validators.append(assertEqualShape(data1=features,
+                                                 data2=times,
+                                                 axes1=0,
+                                                 axes2=0,
+                                                 name1='feature_shape',
+                                                 name2='times',
+                                                 ignore_undetermined=True))
         # Validate electrodes compared to features
-        shape_validators.append(ShapeValidator.assertEqualShape(data1=features,
-                                                                data2=electrodes,
-                                                                axes1=1,
-                                                                axes2=0,
-                                                                name1='feature_shape',
-                                                                name2='electrodes',
-                                                                ignore_undetermined=True))
+        shape_validators.append(assertEqualShape(data1=features,
+                                                 data2=electrodes,
+                                                 axes1=1,
+                                                 axes2=0,
+                                                 name1='feature_shape',
+                                                 name2='electrodes',
+                                                 ignore_undetermined=True))
         # Valided description compared to features
-        shape_validators.append(ShapeValidator.assertEqualShape(data1=features,
-                                                                data2=description,
-                                                                axes1=2,
-                                                                axes2=0,
-                                                                name1='feature_shape',
-                                                                name2='description',
-                                                                ignore_undetermined=True))
+        shape_validators.append(assertEqualShape(data1=features,
+                                                 data2=description,
+                                                 axes1=2,
+                                                 axes2=0,
+                                                 name1='feature_shape',
+                                                 name2='description',
+                                                 ignore_undetermined=True))
         # Raise an error if any of the shapes do not match
         raise_error = False
         error_msg = ""
@@ -426,10 +391,10 @@ class FeatureExtraction(NWBDataInterface):
             if not sv.result:
                 error_msg += sv.message + "\n"
         if raise_error:
-            raise TypeError(error_msg)
+            raise ValueError(error_msg)
 
         # Initialize the object
-        super(FeatureExtraction, self).__init__(source, **kwargs)
+        super(FeatureExtraction, self).__init__(**kwargs)
         self.electrodes = electrodes
         self.description = description
         self.times = list(times)
