@@ -1,10 +1,15 @@
-from collections import Iterable
+try:
+    from collections.abc import Iterable  # Python 3
+except ImportError:
+    from collections import Iterable  # Python 2.7
 
-from .form.utils import docval, popargs, fmt_docval_args
+import numpy as np
+
+from hdmf.utils import docval, popargs, fmt_docval_args, call_docval_func
 
 from . import register_class, CORE_NAMESPACE
 from .base import TimeSeries, _default_resolution, _default_conversion
-from .core import NWBContainer
+from .core import NWBContainer, DynamicTable, ElementIdentifiers
 from .device import Device
 
 
@@ -24,7 +29,6 @@ class IntracellularElectrode(NWBContainer):
 
     @docval({'name': 'name', 'type': str, 'doc': 'the name of this electrode'},
             {'name': 'device', 'type': Device, 'doc': 'the device that was used to record from this electrode'},
-            {'name': 'source', 'type': str, 'doc': 'the source of the data'},
             {'name': 'description', 'type': str,
              'doc': 'Recording description, description of electrode (e.g.,  whole-cell, sharp, etc) \
              COMMENT: Free-form text (can be from Methods)'},
@@ -61,16 +65,12 @@ class PatchClampSeries(TimeSeries):
 
     __nwbfields__ = ('electrode',
                      'gain',
-                     'stimulus_description')
+                     'stimulus_description',
+                     'sweep_number')
 
-    _ancestry = "TimeSeries,PatchClampSeries"
     _help = "Superclass definition for patch-clamp data."
 
     @docval({'name': 'name', 'type': str, 'doc': 'The name of this TimeSeries dataset'},
-            {'name': 'source', 'type': str,
-             'doc': ('Name of TimeSeries or Modules that serve as the source for the data '
-                     'contained here. It can also be the name of a device, for stimulus or '
-                     'acquisition data')},
             {'name': 'data', 'type': ('array_data', 'data', TimeSeries),
              'doc': 'The data this TimeSeries dataset stores. Can also store binary data e.g. image frames'},
             {'name': 'unit', 'type': str, 'doc': 'The base unit of measurement (should be SI unit)'},
@@ -85,7 +85,7 @@ class PatchClampSeries(TimeSeries):
              'doc': 'The smallest meaningful difference (in specified unit) between values in data',
              'default': _default_resolution},
             {'name': 'conversion', 'type': float,
-             'doc': 'Scalar to multiply each element by to conver to volts', 'default': _default_conversion},
+             'doc': 'Scalar to multiply each element by to convert to volts', 'default': _default_conversion},
 
             {'name': 'timestamps', 'type': ('array_data', 'data', TimeSeries),
              'doc': 'Timestamps for samples stored in data', 'default': None},
@@ -100,17 +100,26 @@ class PatchClampSeries(TimeSeries):
              'doc': 'Numerical labels that apply to each element in data', 'default': None},
             {'name': 'control_description', 'type': Iterable,
              'doc': 'Description of each control value', 'default': None},
+            {'name': 'sweep_number', 'type': (int, 'uint64'),
+             'doc': 'Sweep number, allows for grouping different PatchClampSeries together \
+                     via the sweep_table', 'default': None},
             {'name': 'parent', 'type': 'NWBContainer',
              'doc': 'The parent NWBContainer for this NWBContainer', 'default': None})
     def __init__(self, **kwargs):
-        name, source, data, unit, stimulus_description = popargs('name', 'source', 'data',
-                                                                 'unit', 'stimulus_description', kwargs)
-        electrode, gain = popargs('electrode', 'gain', kwargs)
-        super(PatchClampSeries, self).__init__(name, source, data, unit, **kwargs)
+        name, data, unit, stimulus_description = popargs('name', 'data',
+                                                         'unit', 'stimulus_description', kwargs)
+        electrode, gain, sweep_number = popargs('electrode', 'gain', 'sweep_number', kwargs)
+        super(PatchClampSeries, self).__init__(name, data, unit, **kwargs)
         self.electrode = electrode
         self.gain = gain
         self.stimulus_description = stimulus_description
 
+        if sweep_number is not None:
+            if not (sweep_number >= 0):
+                raise ValueError("sweep_number must be a non-negative integer")
+
+            self.sweep_number = np.uint64(sweep_number)
+
 
 @register_class('CurrentClampSeries', CORE_NAMESPACE)
 class CurrentClampSeries(PatchClampSeries):
@@ -124,14 +133,9 @@ class CurrentClampSeries(PatchClampSeries):
                      'bridge_balance',
                      'capacitance_compensation')
 
-    _ancestry = "TimeSeries,PatchClampSeries,CurrentClampSeries"
     _help = "Voltage recorded from cell during current-clamprecording."
 
     @docval({'name': 'name', 'type': str, 'doc': 'The name of this TimeSeries dataset'},
-            {'name': 'source', 'type': str,
-             'doc': ('Name of TimeSeries or Modules that serve as the source for the data '
-                     'contained here. It can also be the name of a device, for stimulus or '
-                     'acquisition data')},
             {'name': 'data', 'type': ('array_data', 'data', TimeSeries),
              'doc': 'The data this TimeSeries dataset stores. Can also store binary data e.g. image frames'},
             {'name': 'unit', 'type': str, 'doc': 'The base unit of measurement (should be SI unit)'},
@@ -142,9 +146,9 @@ class CurrentClampSeries(PatchClampSeries):
             {'name': 'gain', 'type': float, 'doc': 'Units: Volt/Amp (v-clamp) or Volt/Volt (c-clamp)'},
             {'name': 'stimulus_description', 'type': str, 'doc': 'the stimulus name/protocol', 'default': "NA"},
 
-            {'name': 'bias_current', 'type': float, 'doc': 'Unit: Amp'},
-            {'name': 'bridge_balance', 'type': float, 'doc': 'Unit: Ohm'},
-            {'name': 'capacitance_compensation', 'type': float, 'doc': 'Unit: Farad'},
+            {'name': 'bias_current', 'type': float, 'doc': 'Unit: Amp', 'default': None},
+            {'name': 'bridge_balance', 'type': float, 'doc': 'Unit: Ohm', 'default': None},
+            {'name': 'capacitance_compensation', 'type': float, 'doc': 'Unit: Farad', 'default': None},
 
             {'name': 'resolution', 'type': float,
              'doc': 'The smallest meaningful difference (in specified unit) between values in data',
@@ -152,8 +156,6 @@ class CurrentClampSeries(PatchClampSeries):
             {'name': 'conversion', 'type': float,
              'doc': 'Scalar to multiply each element by to conver to volts', 'default': _default_conversion},
 
-            {'name': 'timestamps', 'type': ('array_data', 'data', TimeSeries),
-             'doc': 'Timestamps for samples stored in data', 'default': None},
             {'name': 'timestamps', 'type': ('array_data', 'data', TimeSeries),
              'doc': 'Timestamps for samples stored in data', 'default': None},
             {'name': 'starting_time', 'type': float, 'doc': 'The timestamp of the first sample', 'default': None},
@@ -167,14 +169,17 @@ class CurrentClampSeries(PatchClampSeries):
              'doc': 'Numerical labels that apply to each element in data', 'default': None},
             {'name': 'control_description', 'type': Iterable,
              'doc': 'Description of each control value', 'default': None},
+            {'name': 'sweep_number', 'type': (int, 'uint64'),
+             'doc': 'Sweep number, allows for grouping different PatchClampSeries together \
+                     via the sweep_table', 'default': None},
             {'name': 'parent', 'type': 'NWBContainer',
              'doc': 'The parent NWBContainer for this NWBContainer', 'default': None})
     def __init__(self, **kwargs):
-        name, source, data, unit = popargs('name', 'source', 'data', 'unit', kwargs)
+        name, data, unit = popargs('name', 'data', 'unit', kwargs)
         electrode, gain = popargs('electrode', 'gain', kwargs)
         bias_current, bridge_balance, capacitance_compensation = popargs(
             'bias_current', 'bridge_balance', 'capacitance_compensation', kwargs)
-        super(CurrentClampSeries, self).__init__(name, source, data, unit, electrode, gain, **kwargs)
+        super(CurrentClampSeries, self).__init__(name, data, unit, electrode, gain, **kwargs)
         self.bias_current = bias_current
         self.bridge_balance = bridge_balance
         self.capacitance_compensation = capacitance_compensation
@@ -191,14 +196,9 @@ class IZeroClampSeries(CurrentClampSeries):
 
     __nwbfields__ = ()
 
-    _ancestry = "TimeSeries,PatchClampSeries,CurrentClampSeries,IZeroClampSeries"
     _help = "Voltage from intracellular recordings when all current and amplifier settings are off,"
 
     @docval({'name': 'name', 'type': str, 'doc': 'The name of this TimeSeries dataset'},
-            {'name': 'source', 'type': str,
-             'doc': ('Name of TimeSeries or Modules that serve as the source for the data '
-                     'contained here. It can also be the name of a device, for stimulus or '
-                     'acquisition data')},
             {'name': 'data', 'type': ('array_data', 'data', TimeSeries),
              'doc': 'The data this TimeSeries dataset stores. Can also store binary data e.g. image frames'},
             {'name': 'unit', 'type': str, 'doc': 'The base unit of measurement (should be SI unit)'},
@@ -229,14 +229,17 @@ class IZeroClampSeries(CurrentClampSeries):
              'doc': 'Numerical labels that apply to each element in data', 'default': None},
             {'name': 'control_description', 'type': Iterable,
              'doc': 'Description of each control value', 'default': None},
+            {'name': 'sweep_number', 'type': (int, 'uint64'),
+             'doc': 'Sweep number, allows for grouping different PatchClampSeries together \
+                     via the sweep_table', 'default': None},
             {'name': 'parent', 'type': 'NWBContainer',
              'doc': 'The parent NWBContainer for this NWBContainer', 'default': None})
     def __init__(self, **kwargs):
-        name, source, data, unit = popargs('name', 'source', 'data', 'unit', kwargs)
+        name, data, unit = popargs('name', 'data', 'unit', kwargs)
         electrode, gain = popargs('electrode', 'gain', kwargs)
         bias_current, bridge_balance, capacitance_compensation = popargs(
             'bias_current', 'bridge_balance', 'capacitance_compensation', kwargs)
-        super(IZeroClampSeries, self).__init__(name, source, data, unit, electrode, gain, bias_current,
+        super(IZeroClampSeries, self).__init__(name, data, unit, electrode, gain, bias_current,
                                                bridge_balance, capacitance_compensation, **kwargs)
 
 
@@ -249,14 +252,9 @@ class CurrentClampStimulusSeries(PatchClampSeries):
 
     __nwbfields__ = ()
 
-    _ancestry = "TimeSeries,PatchClampSeries,CurrentClampStimulusSeries"
     _help = "Stimulus current applied during current clamp recording."
 
     @docval({'name': 'name', 'type': str, 'doc': 'The name of this TimeSeries dataset'},
-            {'name': 'source', 'type': str,
-             'doc': ('Name of TimeSeries or Modules that serve as the source for the data '
-                     'contained here. It can also be the name of a device, for stimulus or '
-                     'acquisition data')},
             {'name': 'data', 'type': ('array_data', 'data', TimeSeries),
              'doc': 'The data this TimeSeries dataset stores. Can also store binary data e.g. image frames'},
             {'name': 'unit', 'type': str, 'doc': 'The base unit of measurement (should be SI unit)'},
@@ -285,12 +283,15 @@ class CurrentClampStimulusSeries(PatchClampSeries):
              'doc': 'Numerical labels that apply to each element in data', 'default': None},
             {'name': 'control_description', 'type': Iterable,
              'doc': 'Description of each control value', 'default': None},
+            {'name': 'sweep_number', 'type': (int, 'uint64'),
+             'doc': 'Sweep number, allows for grouping different PatchClampSeries together \
+                     via the sweep_table', 'default': None},
             {'name': 'parent', 'type': 'NWBContainer',
              'doc': 'The parent NWBContainer for this NWBContainer', 'default': None})
     def __init__(self, **kwargs):
-        name, source, data, unit = popargs('name', 'source', 'data', 'unit', kwargs)
+        name, data, unit = popargs('name', 'data', 'unit', kwargs)
         electrode, gain = popargs('electrode', 'gain', kwargs)
-        super(CurrentClampStimulusSeries, self).__init__(name, source, data, unit, electrode, gain, **kwargs)
+        super(CurrentClampStimulusSeries, self).__init__(name, data, unit, electrode, gain, **kwargs)
 
 
 @register_class('VoltageClampSeries', CORE_NAMESPACE)
@@ -309,14 +310,9 @@ class VoltageClampSeries(PatchClampSeries):
                      'whole_cell_capacitance_comp',
                      'whole_cell_series_resistance_comp')
 
-    _ancestry = "TimeSeries,PatchClampSeries,VoltageClampSeries"
     _help = "Current recorded from cell during voltage-clamp recording"
 
     @docval({'name': 'name', 'type': str, 'doc': 'The name of this TimeSeries dataset'},
-            {'name': 'source', 'type': str,
-             'doc': ('Name of TimeSeries or Modules that serve as the source for the data '
-                     'contained here. It can also be the name of a device, for stimulus or '
-                     'acquisition data')},
             {'name': 'data', 'type': ('array_data', 'data', TimeSeries),
              'doc': 'The data this TimeSeries dataset stores. Can also store binary data e.g. image frames'},
             {'name': 'unit', 'type': str, 'doc': 'The base unit of measurement (should be SI unit)'},
@@ -326,13 +322,13 @@ class VoltageClampSeries(PatchClampSeries):
              apply or record this data.'},
             {'name': 'gain', 'type': float, 'doc': 'Units: Volt/Amp (v-clamp) or Volt/Volt (c-clamp)'},
             {'name': 'stimulus_description', 'type': str, 'doc': 'the stimulus name/protocol', 'default': "NA"},
-            {'name': 'capacitance_fast', 'type': float, 'doc': 'Unit: Farad'},
-            {'name': 'capacitance_slow', 'type': float, 'doc': 'Unit: Farad'},
-            {'name': 'resistance_comp_bandwidth', 'type': float, 'doc': 'Unit: Hz'},
-            {'name': 'resistance_comp_correction', 'type': float, 'doc': 'Unit: %'},
-            {'name': 'resistance_comp_prediction', 'type': float, 'doc': 'Unit: %'},
-            {'name': 'whole_cell_capacitance_comp', 'type': float, 'doc': 'Unit: Farad'},
-            {'name': 'whole_cell_series_resistance_comp', 'type': float, 'doc': 'Unit: Ohm'},
+            {'name': 'capacitance_fast', 'type': float, 'doc': 'Unit: Farad', 'default': None},
+            {'name': 'capacitance_slow', 'type': float, 'doc': 'Unit: Farad', 'default': None},
+            {'name': 'resistance_comp_bandwidth', 'type': float, 'doc': 'Unit: Hz', 'default': None},
+            {'name': 'resistance_comp_correction', 'type': float, 'doc': 'Unit: %', 'default': None},
+            {'name': 'resistance_comp_prediction', 'type': float, 'doc': 'Unit: %', 'default': None},
+            {'name': 'whole_cell_capacitance_comp', 'type': float, 'doc': 'Unit: Farad', 'default': None},
+            {'name': 'whole_cell_series_resistance_comp', 'type': float, 'doc': 'Unit: Ohm', 'default': None},
             {'name': 'resolution', 'type': float,
              'doc': 'The smallest meaningful difference (in specified unit) between values in data',
              'default': _default_resolution},
@@ -351,17 +347,20 @@ class VoltageClampSeries(PatchClampSeries):
              'doc': 'Numerical labels that apply to each element in data', 'default': None},
             {'name': 'control_description', 'type': Iterable,
              'doc': 'Description of each control value', 'default': None},
+            {'name': 'sweep_number', 'type': (int, 'uint64'),
+             'doc': 'Sweep number, allows for grouping different PatchClampSeries together \
+                     via the sweep_table', 'default': None},
             {'name': 'parent', 'type': 'NWBContainer',
              'doc': 'The parent NWBContainer for this NWBContainer', 'default': None})
     def __init__(self, **kwargs):
-        name, source, data, unit = popargs('name', 'source', 'data', 'unit', kwargs)
+        name, data, unit = popargs('name', 'data', 'unit', kwargs)
         electrode, gain = popargs('electrode', 'gain', kwargs)
         capacitance_fast, capacitance_slow, resistance_comp_bandwidth, resistance_comp_correction, \
             resistance_comp_prediction, whole_cell_capacitance_comp, whole_cell_series_resistance_comp = popargs(
                 'capacitance_fast', 'capacitance_slow', 'resistance_comp_bandwidth',
                 'resistance_comp_correction', 'resistance_comp_prediction', 'whole_cell_capacitance_comp',
                 'whole_cell_series_resistance_comp', kwargs)
-        super(VoltageClampSeries, self).__init__(name, source, data, unit, electrode, gain, **kwargs)
+        super(VoltageClampSeries, self).__init__(name, data, unit, electrode, gain, **kwargs)
         self.capacitance_fast = capacitance_fast
         self.capacitance_slow = capacitance_slow
         self.resistance_comp_bandwidth = resistance_comp_bandwidth
@@ -380,14 +379,9 @@ class VoltageClampStimulusSeries(PatchClampSeries):
 
     __nwbfields__ = ()
 
-    _ancestry = "TimeSeries,PatchClampSeries,VoltageClampStimulusSeries"
     _help = "Stimulus voltage applied during voltage clamp recording."
 
     @docval({'name': 'name', 'type': str, 'doc': 'The name of this TimeSeries dataset'},
-            {'name': 'source', 'type': str,
-             'doc': ('Name of TimeSeries or Modules that serve as the source for the data '
-                     'contained here. It can also be the name of a device, for stimulus or '
-                     'acquisition data')},
             {'name': 'data', 'type': ('array_data', 'data', TimeSeries),
              'doc': 'The data this TimeSeries dataset stores. Can also store binary data e.g. image frames'},
             {'name': 'unit', 'type': str, 'doc': 'The base unit of measurement (should be SI unit)'},
@@ -414,9 +408,74 @@ class VoltageClampStimulusSeries(PatchClampSeries):
              'doc': 'Numerical labels that apply to each element in data', 'default': None},
             {'name': 'control_description', 'type': Iterable,
              'doc': 'Description of each control value', 'default': None},
+            {'name': 'sweep_number', 'type': (int, 'uint64'),
+             'doc': 'Sweep number, allows for grouping different PatchClampSeries together \
+                     via the sweep_table', 'default': None},
             {'name': 'parent', 'type': 'NWBContainer',
              'doc': 'The parent NWBContainer for this NWBContainer', 'default': None})
     def __init__(self, **kwargs):
-        name, source, data, unit = popargs('name', 'source', 'data', 'unit', kwargs)
+        name, data, unit = popargs('name', 'data', 'unit', kwargs)
         electrode, gain = popargs('electrode', 'gain', kwargs)
-        super(VoltageClampStimulusSeries, self).__init__(name, source, data, unit, electrode, gain, **kwargs)
+        super(VoltageClampStimulusSeries, self).__init__(name, data, unit, electrode, gain, **kwargs)
+
+
+@register_class('SweepTable', CORE_NAMESPACE)
+class SweepTable(DynamicTable):
+    """
+    A SweepTable allows to group PatchClampSeries together which stem from the same sweep.
+    """
+
+    __columns__ = (
+            {'name': 'series', 'description': 'PatchClampSeries with the same sweep number',
+             'required': True, 'index': True},
+            {'name': 'sweep_number', 'description': 'Sweep number of the entries in that row', 'required': True}
+    )
+
+    @docval({'name': 'name', 'type': str, 'doc': 'name of this SweepTable', 'default': 'sweep_table'},
+            {'name': 'description', 'type': str, 'doc': 'Description of this SweepTable',
+             'default': "A sweep table groups different PatchClampSeries together."},
+            {'name': 'id', 'type': ('array_data', ElementIdentifiers), 'doc': 'the identifiers for this table',
+             'default': None},
+            {'name': 'columns', 'type': (tuple, list), 'doc': 'the columns in this table', 'default': None},
+            {'name': 'colnames', 'type': 'array_data', 'doc': 'the names of the columns in this table',
+             'default': None})
+    def __init__(self, **kwargs):
+        call_docval_func(super(SweepTable, self).__init__, kwargs)
+
+    @docval({'name': 'pcs', 'type': PatchClampSeries, 'doc': 'PatchClampSeries to add to the table ' +
+            'must have a valid sweep_number'})
+    def add_entry(self, pcs):
+        """
+        Add the passed PatchClampSeries to the sweep table.
+        """
+
+        kwargs = {'sweep_number': pcs.sweep_number, 'series': [pcs]}
+
+        # FIXME appending to an existing entry would be nicer
+        # but this seems to be not possible
+        self.add_row(**kwargs)
+
+    def get_series(self, sweep_number):
+        """
+        Return a list of PatchClampSeries for the given sweep number.
+        """
+
+        ids = self.__get_row_ids(sweep_number)
+
+        if len(ids) == 0:
+            return None
+
+        matches = []
+
+        for x in ids:
+            for y in self[(x, 'series')]:
+                matches.append(y)
+
+        return matches
+
+    def __get_row_ids(self, sweep_number):
+        """
+        Return the row ids for the given sweep number.
+        """
+
+        return [index for index, elem in enumerate(self['sweep_number'].data) if elem == sweep_number]
