@@ -1,10 +1,14 @@
 import numpy as np
-from collections import Iterable
+import warnings
+try:
+    from collections.abc import Iterable  # Python 3
+except ImportError:
+    from collections import Iterable  # Python 2.7
 
-from .form.utils import docval, popargs, call_docval_func
+from hdmf.utils import docval, popargs, call_docval_func
 
 from . import register_class, CORE_NAMESPACE
-from .base import TimeSeries, _default_resolution, _default_conversion
+from .base import TimeSeries, _default_resolution, _default_conversion, Image
 
 
 @register_class('ImageSeries', CORE_NAMESPACE)
@@ -14,21 +18,15 @@ class ImageSeries(TimeSeries):
     The image data can be stored in the HDF5 file or it will be stored as an external image file.
     '''
 
-    __nwbfields__ = ('bits_per_pixel',
-                     'dimension',
+    __nwbfields__ = ('dimension',
                      'external_file',
                      'starting_frame',
                      'format')
 
-    _ancestry = "TimeSeries,ImageSeries"
     _help = "Storage object for time-series 2-D image data"
 
     @docval({'name': 'name', 'type': str, 'doc': 'The name of this TimeSeries dataset'},
-            {'name': 'source', 'type': str,
-             'doc': ('Name of TimeSeries or Modules that serve as the source for the data '
-                     'contained here. It can also be the name of a device, for stimulus or '
-                     'acquisition data')},
-            {'name': 'data', 'type': ('array_data', 'data', TimeSeries),
+            {'name': 'data', 'type': ('array_data', 'data', TimeSeries), 'shape': ([None] * 3, [None] * 4),
              'doc': 'The data this TimeSeries dataset stores. Can also store binary data e.g. image frames',
              'default': None},
             {'name': 'unit', 'type': str,
@@ -49,8 +47,8 @@ class ImageSeries(TimeSeries):
              'doc': 'The smallest meaningful difference (in specified unit) between values in data',
              'default': _default_resolution},
             {'name': 'conversion', 'type': float,
-             'doc': 'Scalar to multiply each element by to conver to volts', 'default': _default_conversion},
-            {'name': 'timestamps', 'type': ('array_data', 'data', TimeSeries),
+             'doc': 'Scalar to multiply each element by to convert to volts', 'default': _default_conversion},
+            {'name': 'timestamps', 'type': ('array_data', 'data', TimeSeries), 'shape': (None, ),
              'doc': 'Timestamps for samples stored in data', 'default': None},
             {'name': 'starting_time', 'type': float, 'doc': 'The timestamp of the first sample', 'default': None},
             {'name': 'rate', 'type': float, 'doc': 'Sampling rate in Hz', 'default': None},
@@ -76,6 +74,16 @@ class ImageSeries(TimeSeries):
         self.starting_frame = starting_frame
         self.format = format
 
+    @property
+    def bits_per_pixel(self):
+        return self.fields.get('bits_per_pixel')
+
+    @bits_per_pixel.setter
+    def bits_per_pixel(self, val):
+        if val is not None:
+            warnings.warn("bits_per_pixel is no longer used", DeprecationWarning)
+            self.fields['bits_per_pixel'] = val
+
 
 @register_class('IndexSeries', CORE_NAMESPACE)
 class IndexSeries(TimeSeries):
@@ -89,16 +97,11 @@ class IndexSeries(TimeSeries):
 
     __nwbfields__ = ('indexed_timeseries',)
 
-    _ancestry = "TimeSeries,IndexSeries"
     _help = "A sequence that is generated from an existing image stack. Frames can be presented in \
     an arbitrary order. The data[] field stores frame number in reference stack."
 
     @docval({'name': 'name', 'type': str, 'doc': 'The name of this TimeSeries dataset'},
-            {'name': 'source', 'type': str,
-             'doc': ('Name of TimeSeries or Modules that serve as the source for the data '
-                     'contained here. It can also be the name of a device, for stimulus or '
-                     'acquisition data')},
-            {'name': 'data', 'type': ('array_data', 'data', TimeSeries),
+            {'name': 'data', 'type': ('array_data', 'data', TimeSeries), 'shape': (None, ),
              'doc': 'The data this TimeSeries dataset stores. Can also store binary data e.g. image frames'},
             {'name': 'unit', 'type': str, 'doc': 'The base unit of measurement (should be SI unit)'},
 
@@ -124,9 +127,9 @@ class IndexSeries(TimeSeries):
             {'name': 'parent', 'type': 'NWBContainer',
              'doc': 'The parent NWBContainer for this NWBContainer', 'default': None})
     def __init__(self, **kwargs):
-        name, source, data, unit = popargs('name', 'source', 'data', 'unit', kwargs)
+        name, data, unit = popargs('name', 'data', 'unit', kwargs)
         indexed_timeseries = popargs('indexed_timeseries', kwargs)
-        super(IndexSeries, self).__init__(name, source, data, unit, **kwargs)
+        super(IndexSeries, self).__init__(name, data, unit, **kwargs)
         self.indexed_timeseries = indexed_timeseries
 
 
@@ -141,14 +144,9 @@ class ImageMaskSeries(ImageSeries):
 
     __nwbfields__ = ('masked_imageseries',)
 
-    _ancestry = "TimeSeries,ImageSeries,ImageMaskSeries"
     _help = "An alpha mask that is applied to a presented visual stimulus."
 
     @docval({'name': 'name', 'type': str, 'doc': 'The name of this TimeSeries dataset'},
-            {'name': 'source', 'type': str,
-             'doc': ('Name of TimeSeries or Modules that serve as the source for the data '
-                     'contained here. It can also be the name of a device, for stimulus or '
-                     'acquisition data')},
             {'name': 'data', 'type': ('array_data', 'data', TimeSeries),
              'doc': 'The data this TimeSeries dataset stores. Can also store binary data e.g. image frames'},
             {'name': 'unit', 'type': str, 'doc': 'The base unit of measurement (should be SI unit)'},
@@ -186,11 +184,10 @@ class ImageMaskSeries(ImageSeries):
             {'name': 'parent', 'type': 'NWBContainer',
              'doc': 'The parent NWBContainer for this NWBContainer', 'default': None})
     def __init__(self, **kwargs):
-        name, source, data, unit, external_file, starting_frame, format = popargs(
-            'name', 'source', 'data', 'unit', 'external_file', 'starting_frame', 'format', kwargs)
+        name, data, unit, external_file, starting_frame, format = popargs(
+            'name', 'data', 'unit', 'external_file', 'starting_frame', 'format', kwargs)
         masked_imageseries = popargs('masked_imageseries', kwargs)
-        super(ImageMaskSeries, self).__init__(name=name, source=source,
-                                              data=data, unit=unit,
+        super(ImageMaskSeries, self).__init__(name=name, data=data, unit=unit,
                                               external_file=external_file,
                                               starting_frame=starting_frame, format=format, **kwargs)
         self.masked_imageseries = masked_imageseries
@@ -210,21 +207,16 @@ class OpticalSeries(ImageSeries):
                      'field_of_view',
                      'orientation')
 
-    _ancestry = "TimeSeries,ImageSeries,OpticalSeries"
     _help = "Time-series image stack for optical recording or stimulus."
 
     @docval({'name': 'name', 'type': str, 'doc': 'The name of this TimeSeries dataset'},
-            {'name': 'source', 'type': str,
-             'doc': ('Name of TimeSeries or Modules that serve as the source for the data '
-                     'contained here. It can also be the name of a device, for stimulus or '
-                     'acquisition data')},
             {'name': 'data', 'type': ('array_data', 'data', TimeSeries),
              'doc': 'The data this TimeSeries dataset stores. Can also store binary data e.g. image frames'},
             {'name': 'unit', 'type': str, 'doc': 'The base unit of measurement (should be SI unit)'},
             {'name': 'format', 'type': str,
              'doc': 'Format of image. Three types: 1) Image format; tiff, png, jpg, etc. 2) external 3) raw.'},
             {'name': 'distance', 'type': float, 'doc': 'Distance from camera/monitor to target/eye.'},
-            {'name': 'field_of_view', 'type': (list, np.ndarray, 'TimeSeries'),
+            {'name': 'field_of_view', 'type': (list, np.ndarray, 'TimeSeries'), 'shape': ((2, ), (3, )),
              'doc': 'Width, height and depth of image, or imaged area (meters).'},
             {'name': 'orientation', 'type': str,
              'doc': 'Description of image relative to some reference frame (e.g., which way is up). \
@@ -259,12 +251,45 @@ class OpticalSeries(ImageSeries):
             {'name': 'parent', 'type': 'NWBContainer',
              'doc': 'The parent NWBContainer for this NWBContainer', 'default': None})
     def __init__(self, **kwargs):
-        name, source, data, unit, external_file, starting_frame, format = popargs(
-            'name', 'source', 'data', 'unit', 'external_file', 'starting_frame', 'format', kwargs)
+        name, data, unit, external_file, starting_frame, format = popargs(
+            'name', 'data', 'unit', 'external_file', 'starting_frame', 'format', kwargs)
         distance, field_of_view, orientation = popargs('distance', 'field_of_view', 'orientation', kwargs)
-        super(OpticalSeries, self).__init__(name=name, source=source, data=data, unit=unit,
+        super(OpticalSeries, self).__init__(name=name, data=data, unit=unit,
                                             external_file=external_file, starting_frame=starting_frame,
                                             format=format, **kwargs)
         self.distance = distance
         self.field_of_view = field_of_view
         self.orientation = orientation
+
+
+@register_class('GrayscaleImage', CORE_NAMESPACE)
+class GrayscaleImage(Image):
+
+    @docval({'name': 'name', 'type': str, 'doc': 'The name of this image'},
+            {'name': 'data', 'type': ('array_data', 'data'), 'doc': 'data of image', 'shape': (None, None)},
+            {'name': 'resolution', 'type': float, 'doc': 'pixels / cm', 'default': None},
+            {'name': 'description', 'type': str, 'doc': 'description of image', 'default': None})
+    def __init__(self, **kwargs):
+        super(GrayscaleImage, self).__init__(**kwargs)
+
+
+@register_class('RGBImage', CORE_NAMESPACE)
+class RGBImage(Image):
+
+    @docval({'name': 'name', 'type': str, 'doc': 'The name of this image'},
+            {'name': 'data', 'type': ('array_data', 'data'), 'doc': 'data of image', 'shape': (None, None, 3)},
+            {'name': 'resolution', 'type': float, 'doc': 'pixels / cm', 'default': None},
+            {'name': 'description', 'type': str, 'doc': 'description of image', 'default': None})
+    def __init__(self, **kwargs):
+        super(RGBImage, self).__init__(**kwargs)
+
+
+@register_class('RGBAImage', CORE_NAMESPACE)
+class RGBAImage(Image):
+
+    @docval({'name': 'name', 'type': str, 'doc': 'The name of this image'},
+            {'name': 'data', 'type': ('array_data', 'data'), 'doc': 'data of image', 'shape': (None, None, 4)},
+            {'name': 'resolution', 'type': float, 'doc': 'pixels / cm', 'default': None},
+            {'name': 'description', 'type': str, 'doc': 'description of image', 'default': None})
+    def __init__(self, **kwargs):
+        super(RGBAImage, self).__init__(**kwargs)
