@@ -1,6 +1,9 @@
-from collections import Iterable
+try:
+    from collections.abc import Iterable  # Python 3
+except ImportError:
+    from collections import Iterable  # Python 2.7
 
-from .form.utils import docval, popargs, fmt_docval_args
+from hdmf.utils import docval, popargs, fmt_docval_args
 
 from . import register_class, CORE_NAMESPACE
 from .core import NWBContainer, NWBDataInterface
@@ -18,7 +21,6 @@ class AImage(NWBContainer):
                      'focal_depth')
 
     @docval({'name': 'name', 'type': str, 'doc': 'the name of this axis map'},
-            {'name': 'source', 'type': str, 'doc': 'the source of the data'},
             {'name': 'data', 'type': Iterable, 'doc': 'Data field.'},
             {'name': 'bits_per_pixel', 'type': int,
              'doc': 'Number of bits used to represent each value. This is necessary to determine maximum \
@@ -48,11 +50,11 @@ class AxisMap(NWBContainer):
                      'dimension')
 
     @docval({'name': 'name', 'type': str, 'doc': 'the name of this axis map'},
-            {'name': 'source', 'type': str, 'doc': 'the source of the data'},
-            {'name': 'data', 'type': Iterable, 'doc': 'data field.'},
+            {'name': 'data', 'type': Iterable, 'shape': (None, None), 'doc': 'data field.'},
             {'name': 'field_of_view', 'type': Iterable, 'doc': 'Size of viewing area, in meters.'},
             {'name': 'unit', 'type': str, 'doc': 'Unit that axis data is stored in (e.g., degrees)'},
-            {'name': 'dimension', 'type': Iterable, 'doc': 'Number of rows and columns in the image'})
+            {'name': 'dimension', 'type': Iterable, 'shape': (None, ),
+             'doc': 'Number of rows and columns in the image'})
     def __init__(self, **kwargs):
         data, field_of_view, unit, dimension = popargs('data', 'field_of_view', 'unit', 'dimension', kwargs)
         pargs, pkwargs = fmt_docval_args(super(AxisMap, self).__init__, kwargs)
@@ -85,8 +87,7 @@ class ImagingRetinotopy(NWBDataInterface):
 
     _help = "Intrinsic signal optical imaging or Widefield imaging for measuring retinotopy."
 
-    @docval({'name': 'source', 'type': str, 'doc': 'The source of the data represented in this Module Interface.'},
-            {'name': 'sign_map', 'type': AxisMap,
+    @docval({'name': 'sign_map', 'type': AxisMap,
              'doc': 'Sine of the angle between the direction of the gradient in axis_1 and axis_2.'},
             {'name': 'axis_1_phase_map', 'type': AxisMap,
              'doc': 'Phase response to stimulus on the first measured axis.'},
