@@ -1,6 +1,8 @@
+import unittest2 as unittest
+import numpy as np
 from copy import deepcopy
 
-from pynwb.form.build import GroupBuilder, DatasetBuilder, LinkBuilder, RegionBuilder
+from hdmf.build import GroupBuilder, DatasetBuilder, LinkBuilder, ReferenceBuilder
 
 from pynwb.ophys import (
     ImagingPlane,
@@ -23,11 +25,11 @@ class TestImagingPlaneIO(base.TestMapRoundTrip):
     """
 
     def setUpContainer(self):
-        self.device = Device(name='dev1', source='a test source')
-        self.optical_channel = OpticalChannel('optchan1', 'unit test TestImagingPlaneIO',
+        self.device = Device(name='dev1')
+        self.optical_channel = OpticalChannel('optchan1',
                                               'a fake OpticalChannel', 500.)
-        return ImagingPlane('imgpln1', 'unit test TestImagingPlaneIO', self.optical_channel,
-                            'a fake ImagingPlane', self.device, 600., '2.718', 'GFP', 'somewhere in the brain')
+        return ImagingPlane('imgpln1', self.optical_channel, 'a fake ImagingPlane', self.device,
+                            600., 300., 'GFP', 'somewhere in the brain', reference_frame='unknonwn')
 
     def setUpBuilder(self):
         optchan_builder = GroupBuilder(
@@ -35,8 +37,7 @@ class TestImagingPlaneIO(base.TestMapRoundTrip):
             attributes={
                 'neurodata_type': 'OpticalChannel',
                 'namespace': 'core',
-                'help': 'Metadata about an optical channel used to record from an imaging plane',
-                'source': 'unit test TestImagingPlaneIO'},
+                'help': 'Metadata about an optical channel used to record from an imaging plane'},
             datasets={
                 'description': DatasetBuilder('description', 'a fake OpticalChannel'),
                 'emission_lambda': DatasetBuilder('emission_lambda', 500.)},
@@ -44,19 +45,17 @@ class TestImagingPlaneIO(base.TestMapRoundTrip):
         device_builder = GroupBuilder('dev1',
                                       attributes={'neurodata_type': 'Device',
                                                   'namespace': 'core',
-                                                  'help': 'A recording device e.g. amplifier',
-                                                  'source': 'a test source'})
+                                                  'help': 'A recording device e.g. amplifier'})
         return GroupBuilder(
             'imgpln1',
             attributes={
                 'neurodata_type': 'ImagingPlane',
                 'namespace': 'core',
-                'source': 'unit test TestImagingPlaneIO',
                 'help': 'Metadata about an imaging plane'},
             datasets={
                 'description': DatasetBuilder('description', 'a fake ImagingPlane'),
                 'excitation_lambda': DatasetBuilder('excitation_lambda', 600.),
-                'imaging_rate': DatasetBuilder('imaging_rate', '2.718'),
+                'imaging_rate': DatasetBuilder('imaging_rate', 300.),
                 'indicator': DatasetBuilder('indicator', 'GFP'),
                 'location': DatasetBuilder('location', 'somewhere in the brain')},
             groups={
@@ -79,20 +78,18 @@ class TestImagingPlaneIO(base.TestMapRoundTrip):
 
 class TestTwoPhotonSeries(base.TestDataInterfaceIO):
 
-    def make_imaging_plane(self, source):
-        self.device = Device(name='dev1', source='a test source')
-        self.optical_channel = OpticalChannel('optchan1', source, 'a fake OpticalChannel', 500.)
-        self.imaging_plane = ImagingPlane('imgpln1', source, self.optical_channel,
-                                          'a fake ImagingPlane',
-                                          self.device, 600., '2.718', 'GFP', 'somewhere in the brain')
+    def make_imaging_plane(self):
+        self.device = Device(name='dev1')
+        self.optical_channel = OpticalChannel('optchan1', 'a fake OpticalChannel', 500.)
+        self.imaging_plane = ImagingPlane('imgpln1', self.optical_channel, 'a fake ImagingPlane', self.device,
+                                          600., 300., 'GFP', 'somewhere in the brain', reference_frame='unknown')
 
     def setUpContainer(self):
-        self.make_imaging_plane('unit test TestTwoPhotonSeries')
-        data = list(zip(range(10), range(10, 20)))
+        self.make_imaging_plane()
+        data = [[[1., 1.] * 2] * 2]
         timestamps = list(map(lambda x: x/10, range(10)))
         fov = [2.0, 2.0, 5.0]
-        ret = TwoPhotonSeries('test_2ps', 'unit test TestTwoPhotonSeries',
-                              self.imaging_plane, data, 'image_unit', 'raw', fov, 1.7, 3.4,
+        ret = TwoPhotonSeries('test_2ps', self.imaging_plane, data, 'image_unit', 'raw', fov, 1.7, 3.4,
                               timestamps=timestamps, dimension=[2])
         return ret
 
@@ -102,8 +99,7 @@ class TestTwoPhotonSeries(base.TestDataInterfaceIO):
             attributes={
                  'neurodata_type': 'OpticalChannel',
                  'namespace': 'core',
-                 'help': 'Metadata about an optical channel used to record from an imaging plane',
-                 'source': 'unit test TestTwoPhotonSeries'},
+                 'help': 'Metadata about an optical channel used to record from an imaging plane'},
             datasets={
                  'description': DatasetBuilder('description', 'a fake OpticalChannel'),
                  'emission_lambda': DatasetBuilder('emission_lambda', 500.)},
@@ -111,19 +107,17 @@ class TestTwoPhotonSeries(base.TestDataInterfaceIO):
         device_builder = GroupBuilder('dev1',
                                       attributes={'neurodata_type': 'Device',
                                                   'namespace': 'core',
-                                                  'help': 'A recording device e.g. amplifier',
-                                                  'source': 'a test source'})
+                                                  'help': 'A recording device e.g. amplifier'})
         imgpln_builder = GroupBuilder(
             'imgpln1',
             attributes={
                 'neurodata_type': 'ImagingPlane',
                 'namespace': 'core',
-                'help': 'Metadata about an imaging plane',
-                'source': 'unit test TestTwoPhotonSeries'},
+                'help': 'Metadata about an imaging plane'},
             datasets={
                 'description': DatasetBuilder('description', 'a fake ImagingPlane'),
                 'excitation_lambda': DatasetBuilder('excitation_lambda', 600.),
-                'imaging_rate': DatasetBuilder('imaging_rate', '2.718'),
+                'imaging_rate': DatasetBuilder('imaging_rate', 300.),
                 'indicator': DatasetBuilder('indicator', 'GFP'),
                 'location': DatasetBuilder('location', 'somewhere in the brain')},
             groups={
@@ -134,14 +128,13 @@ class TestTwoPhotonSeries(base.TestDataInterfaceIO):
             }
         )
 
-        data = list(zip(range(10), range(10, 20)))
+        data = [[[1., 1.] * 2] * 2]
         timestamps = list(map(lambda x: x/10, range(10)))
         return GroupBuilder(
             'test_2ps',
             attributes={
-                'source': 'unit test TestTwoPhotonSeries',
                 'pmt_gain':  1.7,
-                'scan_line_rate':  3.4,
+                'scan_line_rate':  np.float32(3.4),
                 'namespace': base.CORE_NAMESPACE,
                 'comments': 'no comments',
                 'description': 'no description',
@@ -182,27 +175,29 @@ class TestPlaneSegmentation(base.TestMapRoundTrip):
                     (7, 8, 2.0), (9, 10, 2.)]
 
         ts = [0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]
-        self.image_series = ImageSeries(name='test_iS', source='a hypothetical source', dimension=[2],
+        self.image_series = ImageSeries(name='test_iS', dimension=[2],
                                         external_file=['images.tiff'],
                                         starting_frame=[1, 2, 3], format='tiff', timestamps=ts)
 
-        self.device = Device(name='dev1', source='a test source')
-        self.optical_channel = OpticalChannel('test_optical_channel', 'optical channel source',
+        self.device = Device(name='dev1')
+        self.optical_channel = OpticalChannel('test_optical_channel',
                                               'optical channel description', 500.)
-        self.imaging_plane = ImagingPlane('test_imaging_plane',
-                                          'ophys integration tests',
+        self.imaging_plane = ImagingPlane('imgpln1',
                                           self.optical_channel,
-                                          'imaging plane description',
+                                          'a fake ImagingPlane',
                                           self.device,
-                                          600., '2.718', 'GFP', 'somewhere in the brain',
-                                          (1, 2, 1, 2, 3), 4.0, 'manifold unit', 'A frame to refer to')
+                                          600., 200., 'GFP', 'somewhere in the brain',
+                                          (((1., 2., 3.), (4., 5., 6.)),),
+                                          2., 'a unit',
+                                          reference_frame='unknown')
 
         self.img_mask = deepcopy(img_mask)
         self.pix_mask = deepcopy(pix_mask)
-        pS = PlaneSegmentation('integration test PlaneSegmentation', 'plane segmentation description',
+        self.pxmsk_index = [3, 5]
+        pS = PlaneSegmentation('plane segmentation description',
                                self.imaging_plane, 'test_plane_seg_name', self.image_series)
-        pS.add_roi('1234', pix_mask[0:3], img_mask[0])
-        pS.add_roi('5678', pix_mask[3:5], img_mask[1])
+        pS.add_roi(pixel_mask=pix_mask[0:3], image_mask=img_mask[0])
+        pS.add_roi(pixel_mask=pix_mask[3:5], image_mask=img_mask[1])
         return pS
 
     @staticmethod
@@ -212,8 +207,7 @@ class TestPlaneSegmentation(base.TestMapRoundTrip):
             attributes={
                 'neurodata_type': 'OpticalChannel',
                 'namespace': 'core',
-                'help': 'Metadata about an optical channel used to record from an imaging plane',
-                'source': 'optical channel source'},
+                'help': 'Metadata about an optical channel used to record from an imaging plane'},
             datasets={
                 'description': DatasetBuilder('description', 'optical channel description'),
                 'emission_lambda': DatasetBuilder('emission_lambda', 500.)},
@@ -221,21 +215,19 @@ class TestPlaneSegmentation(base.TestMapRoundTrip):
         device_builder = GroupBuilder('dev1',
                                       attributes={'neurodata_type': 'Device',
                                                   'namespace': 'core',
-                                                  'help': 'A recording device e.g. amplifier',
-                                                  'source': 'a test source'})
+                                                  'help': 'A recording device e.g. amplifier'})
         self.imgpln_builder = GroupBuilder(
             'imgpln1',
             attributes={
                 'neurodata_type': 'ImagingPlane',
                 'namespace': 'core',
-                'source': 'ophys integration tests',
                 'help': 'Metadata about an imaging plane'},
             datasets={
                 'description': DatasetBuilder('description', 'imaging plane description'),
                 'excitation_lambda': DatasetBuilder('excitation_lambda', 600.),
-                'imaging_rate': DatasetBuilder('imaging_rate', '2.718'),
+                'imaging_rate': DatasetBuilder('imaging_rate', 300.),
                 'indicator': DatasetBuilder('indicator', 'GFP'),
-                'manifold': DatasetBuilder('manifold', (1, 2, 1, 2, 3),
+                'manifold': DatasetBuilder('manifold', (((1., 2., 3.), (4., 5., 6.)),),
                                            attributes={'conversion': 4.0, 'unit': 'manifold unit'}),
                 'reference_frame': DatasetBuilder('reference_frame', 'A frame to refer to'),
                 'location': DatasetBuilder('location', 'somewhere in the brain')},
@@ -248,8 +240,7 @@ class TestPlaneSegmentation(base.TestMapRoundTrip):
         )
         ts = [0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]
         self.is_builder = GroupBuilder('test_iS',
-                                       attributes={'source': 'a hypothetical source',
-                                                   'namespace': 'core',
+                                       attributes={'namespace': 'core',
                                                    'neurodata_type': 'ImageSeries',
                                                    'description': 'no description',
                                                    'comments': 'no comments',
@@ -264,40 +255,43 @@ class TestPlaneSegmentation(base.TestMapRoundTrip):
                                                  'dimension': DatasetBuilder('dimension', [2]),
                                                  })
 
-        self.pixel_masks_builder = DatasetBuilder('pixel_masks', self.pix_mask,
+        self.pixel_masks_builder = DatasetBuilder('pixel_mask', self.pix_mask,
                                                   attributes={
                                                    'namespace': 'core',
-                                                   'neurodata_type': 'PixelMasks',
-                                                   'help': 'a concatenated array of pixel masks'})
+                                                   'neurodata_type': 'VectorData',
+                                                   'description': 'Pixel masks for each ROI',
+                                                   'help': 'Values for a list of elements'})
 
-        self.image_masks_builder = DatasetBuilder('image_masks', self.img_mask,
+        self.pxmsk_index_builder = DatasetBuilder('pixel_mask_index', self.pxmsk_index,
                                                   attributes={
                                                    'namespace': 'core',
-                                                   'neurodata_type': 'ImageMasks',
-                                                   'help': 'an array of image masks'})
+                                                   'neurodata_type': 'VectorIndex',
+                                                   'target': ReferenceBuilder(self.pixel_masks_builder),
+                                                   'help': 'indexes into a list of values for a list of elements'})
+
+        self.image_masks_builder = DatasetBuilder('image_mask', self.img_mask,
+                                                  attributes={
+                                                   'namespace': 'core',
+                                                   'neurodata_type': 'VectorData',
+                                                   'description': 'Image masks for each ROI',
+                                                   'help': 'Values for a list of elements'})
 
-        self.rois_builder = DatasetBuilder('rois', [
-                                            ('1234', RegionBuilder(slice(0, 3), self.pixel_masks_builder),
-                                             RegionBuilder([0], self.image_masks_builder)),
-                                            ('5678', RegionBuilder(slice(3, 5), self.pixel_masks_builder),
-                                             RegionBuilder([1], self.image_masks_builder))
-                                        ],
-                                        attributes={
-                                         'namespace': 'core',
-                                         'neurodata_type': 'ROITable',
-                                         'help': 'A table for storing ROI data'})
         ps_builder = GroupBuilder(
             'test_plane_seg_name',
             attributes={
                 'neurodata_type': 'PlaneSegmentation',
                 'namespace': 'core',
-                'source': 'integration test PlaneSegmentation',
+                'description': 'plane segmentation description',
+                'colnames': (b'image_mask', b'pixel_mask'),
                 'help': 'Results from segmentation of an imaging plane'},
             datasets={
-                'description': DatasetBuilder('description', 'plane segmentation description'),
-                'rois': self.rois_builder,
-                'pixel_masks': self.pixel_masks_builder,
-                'image_masks': self.image_masks_builder,
+                'id': DatasetBuilder('id', data=[0, 1],
+                                     attributes={'help': 'unique identifiers for a list of elements',
+                                                 'namespace': 'core',
+                                                 'neurodata_type': 'ElementIdentifiers'}),
+                'pixel_mask': self.pixel_masks_builder,
+                'pixel_mask_index': self.pxmsk_index_builder,
+                'image_mask': self.image_masks_builder,
             },
             groups={
                 'reference_images': GroupBuilder('reference_images', groups={'test_iS': self.is_builder}),
@@ -317,37 +311,80 @@ class TestPlaneSegmentation(base.TestMapRoundTrip):
     def addContainer(self, nwbfile):
         nwbfile.add_device(self.device)
         nwbfile.add_imaging_plane(self.imaging_plane)
-        img_seg = ImageSegmentation('plane segmentation round trip')
+        img_seg = ImageSegmentation()
         img_seg.add_plane_segmentation(self.container)
         mod = nwbfile.create_processing_module('plane_seg_test_module',
-                                               'plane segmentation round trip',
                                                'a plain module for testing')
-        mod.add_data_interface(img_seg)
+        mod.add(img_seg)
 
     def getContainer(self, nwbfile):
         mod = nwbfile.get_processing_module('plane_seg_test_module')
-        img_seg = mod.get_data_interface('ImageSegmentation')
+        img_seg = mod.get('ImageSegmentation')
         return img_seg.get_plane_segmentation('test_plane_seg_name')
 
 
+class MaskRoundTrip(TestPlaneSegmentation):
+
+    def setBoilerPlateObjects(self):
+        ts = [0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]
+        self.image_series = ImageSeries(name='test_iS', dimension=[2],
+                                        external_file=['images.tiff'],
+                                        starting_frame=[1, 2, 3], format='tiff', timestamps=ts)
+        self.device = Device(name='dev1')
+        self.optical_channel = OpticalChannel('test_optical_channel',
+                                              'optical channel description', 500.)
+        self.imaging_plane = ImagingPlane('test_imaging_plane',
+                                          self.optical_channel,
+                                          'imaging plane description',
+                                          self.device,
+                                          600., 300., 'GFP', 'somewhere in the brain',
+                                          (((1., 2., 3.), (4., 5., 6.)),),
+                                          4.0, 'manifold unit', 'A frame to refer to')
+        return PlaneSegmentation('description', self.imaging_plane, 'test_plane_seg_name',
+                                 self.image_series)
+
+    def setUpBuilder(self):
+        raise unittest.SkipTest("no builder")
+
+
+class PixelMaskRoundtrip(MaskRoundTrip):
+
+    def setUpContainer(self):
+        pix_mask = [(1, 2, 1.0), (3, 4, 1.0), (5, 6, 1.0),
+                    (7, 8, 2.0), (9, 10, 2.)]
+        pS = self.setBoilerPlateObjects()
+        pS.add_roi(pixel_mask=pix_mask[0:3])
+        pS.add_roi(pixel_mask=pix_mask[3:5])
+        return pS
+
+
+class ImageMaskRoundtrip(MaskRoundTrip):
+
+    def setUpContainer(self):
+        w, h = 5, 5
+        img_mask = [[[1.0 for x in range(w)] for y in range(h)], [[2.0 for x in range(w)] for y in range(h)]]
+        pS = self.setBoilerPlateObjects()
+        pS.add_roi(image_mask=img_mask[0])
+        pS.add_roi(image_mask=img_mask[1])
+        return pS
+
+
 class TestRoiResponseSeriesIO(base.TestDataInterfaceIO):
 
     def setUpContainer(self):
         self.plane_segmentation = TestPlaneSegmentation.buildPlaneSegmentation(self)
         self.rt_region = self.plane_segmentation.create_roi_table_region('the first of two ROIs', region=[0])
 
-        data = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
+        data = [0., 1., 2., 3., 4., 5., 6., 7., 8., 9.]
         timestamps = [0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]
 
-        return RoiResponseSeries('test_roi_response_series', 'RoiResponseSeries integration test',
-                                 data, 'lumens', self.rt_region, timestamps=timestamps)
+        return RoiResponseSeries('test_roi_response_series', data, 'lumens', self.rt_region, timestamps=timestamps)
 
     def setUpBuilder(self):
-        TestPlaneSegmentation.get_plane_segmentation_builder(self)
+        ps_builder = TestPlaneSegmentation.get_plane_segmentation_builder(self)
         return GroupBuilder(
             'test_roi_response_series',
             attributes={
-                'source': 'RoiResponseSeries integration test',
                 'namespace': base.CORE_NAMESPACE,
                 'comments': 'no comments',
                 'description': 'no description',
@@ -364,20 +401,20 @@ class TestRoiResponseSeriesIO(base.TestDataInterfaceIO):
                 ),
                 'timestamps': DatasetBuilder('timestamps', [0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9],
                                              attributes={'unit': 'Seconds', 'interval': 1}),
-                'rois': DatasetBuilder('rois', RegionBuilder([0], self.rois_builder),
-                                       attributes={'help': 'A region reference to an ROITable',
+                'rois': DatasetBuilder('rois', data=[0],
+                                       attributes={'help': 'a subset (i.e. slice or region) of a DynamicTable',
                                                    'description': 'the first of two ROIs',
+                                                   'table': ReferenceBuilder(ps_builder),
                                                    'namespace': 'core',
-                                                   'neurodata_type': 'ROITableRegion'}),
+                                                   'neurodata_type': 'DynamicTableRegion'}),
             })
 
     def addContainer(self, nwbfile):
         nwbfile.add_device(self.device)
         nwbfile.add_imaging_plane(self.imaging_plane)
-        img_seg = ImageSegmentation('plane segmentation round trip')
+        img_seg = ImageSegmentation()
         img_seg.add_plane_segmentation(self.plane_segmentation)
         mod = nwbfile.create_processing_module('plane_seg_test_module',
-                                               'plane segmentation round trip',
                                                'a plain module for testing')
-        mod.add_data_interface(img_seg)
+        mod.add(img_seg)
         super(TestRoiResponseSeriesIO, self).addContainer(nwbfile)
